@@ -32,8 +32,8 @@ pub enum ParseSyntaxError {
 }
 
 impl fmt::Display for ParseSyntaxError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        use ParseSyntaxError::*;
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        use crate::ParseSyntaxError::*;
 
         match *self {
             RegexCompileError(ref regex, ref error) =>
@@ -46,7 +46,7 @@ impl fmt::Display for ParseSyntaxError {
 
 impl Error for ParseSyntaxError {
     fn description(&self) -> &str {
-        use ParseSyntaxError::*;
+        use crate::ParseSyntaxError::*;
 
         match *self {
             InvalidYaml(_) => "Invalid YAML file syntax",
@@ -60,8 +60,8 @@ impl Error for ParseSyntaxError {
         }
     }
 
-    fn cause(&self) -> Option<&Error> {
-        use ParseSyntaxError::*;
+    fn cause(&self) -> Option<&dyn Error> {
+        use crate::ParseSyntaxError::*;
 
         match *self {
             InvalidYaml(ref error) => Some(error),
@@ -184,7 +184,7 @@ impl SyntaxDefinition {
     }
 
     fn parse_contexts(map: &Hash,
-                      state: &mut ParserState)
+                      state: &mut ParserState<'_>)
                       -> Result<HashMap<String, Context>, ParseSyntaxError> {
         let mut contexts = HashMap::new();
         for (key, value) in map.iter() {
@@ -200,7 +200,7 @@ impl SyntaxDefinition {
 
     fn parse_context(vec: &[Yaml],
                      // TODO: Maybe just pass the scope repo if that's all that's needed?
-                     state: &mut ParserState,
+                     state: &mut ParserState<'_>,
                      contexts: &mut HashMap<String, Context>,
                      is_prototype: bool,
                      namer: &mut ContextNamer)
@@ -254,7 +254,7 @@ impl SyntaxDefinition {
     }
 
     fn parse_reference(y: &Yaml,
-                       state: &mut ParserState,
+                       state: &mut ParserState<'_>,
                        contexts: &mut HashMap<String, Context>,
                        namer: &mut ContextNamer)
                        -> Result<ContextReference, ParseSyntaxError> {
@@ -292,69 +292,22 @@ impl SyntaxDefinition {
         }
     }
 
-    fn resolve_variables(raw_regex: &str, state: &ParserState) -> String {
-        state.variable_regex.replace_all(raw_regex, |caps: &Captures| {
-            let var_regex_raw =
-                state.variables.get(caps.at(1).unwrap_or("")).map_or("", |x| &**x);
-            Self::resolve_variables(var_regex_raw, state)
-        })
-    }
-
-    fn try_compile_regex(regex_str: &str) -> Result<(), ParseSyntaxError> {
-        // Replace backreferences with a placeholder value that will also appear in errors
-        let regex_str = substitute_backrefs_in_regex(regex_str, |i| Some(format!("<placeholder_{}>", i)));
-
-        let result = Regex::with_options(&regex_str,
-                                         RegexOptions::REGEX_OPTION_CAPTURE_GROUP,
-                                         Syntax::default());
-        match result {
-            Err(onig_error) => {
-                Err(ParseSyntaxError::RegexCompileError(regex_str, onig_error))
-            },
-            _ => Ok(())
-        }
-    }
-
     fn parse_match_pattern(map: &Hash,
-                           state: &mut ParserState,
+                           state: &mut ParserState<'_>,
                            contexts: &mut HashMap<String, Context>,
                            namer: &mut ContextNamer)
                            -> Result<MatchPattern, ParseSyntaxError> {
         let raw_regex = get_key(map, "match", |x| x.as_str())?;
-        let regex_str_1 = Self::resolve_variables(raw_regex, state);
-        // if the passed in strings don't include newlines (unlike Sublime) we can't match on them
-        let regex_str = if state.lines_include_newline {
-            regex_str_1
-        } else {
-            rewrite_regex(regex_str_1)
-        };
+        let regex_str = Self::parse_regex(raw_regex, state)?;
         // println!("{:?}", regex_str);
 
-        Self::try_compile_regex(&regex_str)?;
-
         let scope = get_key(map, "scope", |x| x.as_str())
             .ok()
             .map(|s| str_to_scopes(s, state.scope_repo))
             .unwrap_or_else(|| Ok(vec![]))?;
 
-
         let captures = if let Ok(map) = get_key(map, "captures", |x| x.as_hash()) {
-            let r = RegexRewriter {
-                bytes: regex_str.as_bytes(),
-                index: 0,
-            };
-            let valid_indexes = r.get_consuming_capture_indexes();
-            let mut res_map = Vec::new();
-            for (key, value) in map.iter() {
-                if let (Some(key_int), Some(val_str)) = (key.as_i64(), value.as_str()) {
-                    if valid_indexes.contains(&(key_int as usize)) {
-                        res_map.push((key_int as usize,
-                                      str_to_scopes(val_str, state.scope_repo)?));
-                    }
-                }
-            }
-            
-            Some(res_map)
+            Some(Self::parse_captures(map, &regex_str, state)?)
         } else {
             None
         };
@@ -440,7 +393,7 @@ impl SyntaxDefinition {
     }
 
     fn parse_pushargs(y: &Yaml,
-                      state: &mut ParserState,
+                      state: &mut ParserState<'_>,
                       contexts: &mut HashMap<String, Context>,
                       namer: &mut ContextNamer)
                       -> Result<Vec<ContextReference>, ParseSyntaxError> {
@@ -458,6 +411,62 @@ impl SyntaxDefinition {
         }
     }
 
+    fn parse_regex(raw_regex: &str, state: &ParserState<'_>) -> Result<String, ParseSyntaxError> {
+        let mut regex = Self::resolve_variables(raw_regex, state);
+        if !state.lines_include_newline {
+            // If the passed in strings don't include newlines (unlike Sublime) we can't match on
+            // them using the original regex. So this tries to rewrite the regex in a way that
+            // allows matching against lines without newlines (essentially replacing `\n` with `$`).
+            regex = rewrite_regex(regex);
+        };
+        Self::try_compile_regex(&regex)?;
+        Ok(regex)
+    }
+
+    fn resolve_variables(raw_regex: &str, state: &ParserState<'_>) -> String {
+        state.variable_regex.replace_all(raw_regex, |caps: &Captures<'_>| {
+            let var_regex_raw =
+                state.variables.get(caps.at(1).unwrap_or("")).map_or("", |x| &**x);
+            Self::resolve_variables(var_regex_raw, state)
+        })
+    }
+
+    fn try_compile_regex(regex_str: &str) -> Result<(), ParseSyntaxError> {
+        // Replace backreferences with a placeholder value that will also appear in errors
+        let regex_str = substitute_backrefs_in_regex(regex_str, |i| Some(format!("<placeholder_{}>", i)));
+
+        let result = Regex::with_options(&regex_str,
+                                         RegexOptions::REGEX_OPTION_CAPTURE_GROUP,
+                                         Syntax::default());
+        match result {
+            Err(onig_error) => {
+                Err(ParseSyntaxError::RegexCompileError(regex_str, onig_error))
+            },
+            _ => Ok(())
+        }
+    }
+
+    fn parse_captures(
+        map: &Hash,
+        regex_str: &str,
+        state: &mut ParserState<'_>,
+    ) -> Result<CaptureMapping, ParseSyntaxError> {
+        let r = RegexRewriter {
+            bytes: regex_str.as_bytes(),
+            index: 0,
+        };
+        let valid_indexes = r.get_consuming_capture_indexes();
+        let mut captures = Vec::new();
+        for (key, value) in map.iter() {
+            if let (Some(key_int), Some(val_str)) = (key.as_i64(), value.as_str()) {
+                if valid_indexes.contains(&(key_int as usize)) {
+                    captures.push((key_int as usize, str_to_scopes(val_str, state.scope_repo)?));
+                }
+            }
+        }
+        Ok(captures)
+    }
+
     /// Sublime treats the top level context slightly differently from
     /// including the main context from other syntaxes. When main is popped
     /// it is immediately re-added and when it is `set` over the file level
@@ -466,7 +475,7 @@ impl SyntaxDefinition {
     /// See https://github.com/trishume/syntect/issues/58 for more.
     fn add_initial_contexts(
         contexts: &mut HashMap<String, Context>,
-        state: &mut ParserState,
+        state: &mut ParserState<'_>,
         top_level_scope: Scope,
     ) {
         let yaml_docs = YamlLoader::load_from_str(START_CONTEXT).unwrap();
@@ -748,8 +757,8 @@ impl<'a> RegexRewriter<'a> {
 
 #[cfg(test)]
 mod tests {
-    use parsing::syntax_definition::*;
-    use parsing::Scope;
+    use crate::parsing::syntax_definition::*;
+    use crate::parsing::Scope;
     use super::*;
 
     #[test]
@@ -824,7 +833,7 @@ mod tests {
             &Pattern::Match(ref match_pat) => {
                 let m: &CaptureMapping = match_pat.captures.as_ref().expect("test failed");
                 assert_eq!(&m[0], &(1,vec![Scope::new("meta.preprocessor.c++").unwrap()]));
-                use parsing::syntax_definition::ContextReference::*;
+                use crate::parsing::syntax_definition::ContextReference::*;
 
                 // this is sadly necessary because Context is not Eq because of the Regex
                 let expected = MatchOperation::Push(vec![
